#   Copyright 2014 Philip Schwartz
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#


from abc import ABCMeta, abstractmethod, abstractproperty


class ColorMap:
    __metaclass__ = ABCMeta

    @abstractproperty
    def _attribute(self):
        pass

    @abstractproperty
    def _separator(self):
        pass

    @abstractproperty
    def _digit(self):
        pass

    @abstractproperty
    def _color_map(self):
        pass

    @abstractproperty
    def _reset(self):
        pass

    @abstractproperty
    def _clean(self):
        pass

    @abstractmethod
    def color(self):
        pass

    @abstractmethod
    def strip(self):
        pass
